#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr 15 16:08:28 2024

@author: noam
"""

import matplotlib.pyplot as plt
import xarray as xr
import numpy as np
import cmasher as cmr
from scipy.interpolate import RegularGridInterpolator
from matplotlib.gridspec import GridSpec
from matplotlib.ticker import ScalarFormatter

##############################################################################
code = 'VERT'
data = xr.open_dataset('BEAU_' + code + '.nc') 
ts = -1

f0 = np.sin(np.pi/2)*4*np.pi/(86400)
h_all = data.h_av.sum(dim='layers')
h = data.h_inst[ts, :, :, :].where(h_all > 500)
h_av = data.h_av.where(h_all > 500)
zeta = data.zeta_inst[ts, :, :, :].where(data.zeta_inst[ts, :, :, :] != 0)
zeta_av = data.zeta_av.where(data.zeta_av != 0)
u = data.u_inst[ts, :, :, :].where(data.u_inst[ts, :, :, :] != 0)
v = data.v_inst[ts, :, :, :].where(data.v_inst[ts, :, :, :] != 0)
u_av = data.u_av.where(data.u_av != 0)
v_av = data.v_av.where(data.v_av != 0)

##############################################################################
# RELATIVE VORTICITY #########################################################
##############################################################################

f = plt.figure(tight_layout=True, figsize=(7, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))



img = ax[0].contourf(data.xp1/1e3, data.yp1/1e3,
                     zeta[0]/f0, cmap=cmr.fusion_r,
                     levels=np.linspace(-0.4, 0.4, 25), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='--', alpha=0.5)

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Relative vorticity, $\zeta_1/f$ (s$^{-1}$)', fontsize=10)

plt.savefig('relative_vorticity_snapshot_BEAU' + code + '.png', bbox_inches='tight', dpi=400)
plt.close()

##############################################################################
# VELOCITY ###################################################################
##############################################################################

# Regrid vorticity to x, y
u_interp1 = RegularGridInterpolator((data.xp1.data, data.y.data), u.data[0].T)
v_interp1 = RegularGridInterpolator((data.x.data, data.yp1.data), v.data[0].T)
xx, yy = np.meshgrid(data.x.astype(float), data.y.astype(float), indexing='ij')
u1 = u_interp1(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v1 = v_interp1(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed1 = np.sqrt(u1**2 + v1**2) 

u_interp2 = RegularGridInterpolator((data.xp1.data, data.y.data), u.data[1].T)
v_interp2 = RegularGridInterpolator((data.x.data, data.yp1.data), v.data[1].T)
u2 = u_interp2(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v2 = v_interp2(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed2 = np.sqrt(u2**2 + v2**2) 

u_interp3 = RegularGridInterpolator((data.xp1.data, data.y.data), u.data[2].T)
v_interp3 = RegularGridInterpolator((data.x.data, data.yp1.data), v.data[2].T)
u3 = u_interp3(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v3 = v_interp3(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed3 = np.sqrt(u3**2 + v3**2) 

u_av_interp1 = RegularGridInterpolator((data.xp1.data, data.y.data), u_av.data[0].T)
v_av_interp1 = RegularGridInterpolator((data.x.data, data.yp1.data), v_av.data[0].T)
u_av1 = u_av_interp1(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v_av1 = v_av_interp1(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed_av1 = np.sqrt(u_av1**2 + v_av1**2) 

u_av_interp2 = RegularGridInterpolator((data.xp1.data, data.y.data), u_av.data[1].T)
v_av_interp2 = RegularGridInterpolator((data.x.data, data.yp1.data), v_av.data[1].T)
u_av2 = u_av_interp2(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v_av2 = v_av_interp2(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed_av2 = np.sqrt(u_av2**2 + v_av2**2) 

u_av_interp3 = RegularGridInterpolator((data.xp1.data, data.y.data), u_av.data[2].T)
v_av_interp3 = RegularGridInterpolator((data.x.data, data.yp1.data), v_av.data[2].T)
u_av3 = u_av_interp3(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
v_av3 = v_av_interp3(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
speed_av3 = np.sqrt(u_av3**2 + v_av3**2) 

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

f0 = np.sin(np.pi/2)*4*np.pi/(86400)

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     speed1, cmap=cmr.rainforest_r,
                     levels=np.linspace(0, 1, 11), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='--', alpha=0.5)

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Surface layer speed (m s$^{-1}$)', fontsize=10)
        
plt.savefig('velocity_snapshot_BEAU' + code + '.png', bbox_inches='tight', dpi=400)
plt.close()

##############################################################################
# VELOCITY ALT ###############################################################
##############################################################################

inc = 5

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

f0 = np.sin(np.pi/2)*4*np.pi/(86400)

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     speed1, cmap=cmr.rainforest_r,
                     levels=np.linspace(0, 1, 11), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u1[::inc, ::inc], v1[::inc, ::inc],
             color='w')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Surface layer speed (m s$^{-1}$)', fontsize=10)
        
plt.savefig('velocity_alt_snapshot_BEAU' + code + '_L1.png', bbox_inches='tight', dpi=400)
plt.close()

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

f0 = np.sin(np.pi/2)*4*np.pi/(86400)

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     speed2, cmap=cmr.rainforest_r,
                     levels=np.linspace(0, 0.5, 11), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u2[::inc, ::inc], v2[::inc, ::inc],
             color='w')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Surface layer speed (m s$^{-1}$)', fontsize=10)
        
plt.savefig('velocity_alt_snapshot_BEAU' + code + '_L2.png', bbox_inches='tight', dpi=400)
plt.close()


f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

f0 = np.sin(np.pi/2)*4*np.pi/(86400)

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     speed3, cmap=cmr.rainforest_r,
                     levels=np.linspace(0, 0.1, 11), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u3[::inc, ::inc], v3[::inc, ::inc],
             color='w')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Surface layer speed (m s$^{-1}$)', fontsize=10)
        
plt.savefig('velocity_alt_snapshot_BEAU' + code + '_L3.png', bbox_inches='tight', dpi=400)
plt.close()


##############################################################################
# POTENTIAL VORTICITY ########################################################
##############################################################################

# L1
# Regrid vorticity to x, y
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta[0]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h[0]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(2e-7, 2.2e-6, 21), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u1[::inc, ::inc], v1[::inc, ::inc],
             color='k', alpha=0.5, scale=30, width=0.003)
ax[0].quiver(1280, 50, 1, 0, color='k', scale=30, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f + \zeta_1)/h_1$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_snapshot_BEAU' + code + '_L1.png', bbox_inches='tight', dpi=400)
plt.close()

# L2
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta[1]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h[1]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(1.18e-6, 1.3e-6, 25), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u2[::inc, ::inc], v2[::inc, ::inc],
             color='k', alpha=0.5, scale=10, width=0.003)
ax[0].quiver(1230, 50, 1, 0, color='k', scale=10, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f + \zeta_2)/h_2$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_snapshot_BEAU' + code + '_L2.png', bbox_inches='tight', dpi=400)
plt.close()

# L3
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta[2]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h[2]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(3e-8, 2e-7, 18), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u3[::inc, ::inc], v3[::inc, ::inc],
             color='k', alpha=0.5, scale=3, width=0.003)
ax[0].quiver(1280, 50, 0.1, 0, color='k', scale=3, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'0.1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f+\zeta_3)/h_3$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_snapshot_BEAU' + code + '_L3.png', bbox_inches='tight', dpi=400)
plt.close()

##############################################################################
# POTENTIAL VORTICITY (MEAN) #################################################
##############################################################################

# L1
# Regrid vorticity to x, y
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta_av[0]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h_av[0]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(4e-7, 2.4e-6, 21), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u_av1[::inc, ::inc], v_av1[::inc, ::inc],
             color='k', alpha=0.5, scale=30, width=0.003)
ax[0].quiver(1280, 50, 1, 0, color='k', scale=30, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f + \overline{\zeta}_1)/\overline{h}_1$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_mean_BEAU' + code + '_L1.png', bbox_inches='tight', dpi=400)
plt.close()

# L2
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta_av[1]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h_av[1]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(1.21e-6, 1.27e-6, 21), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u_av2[::inc, ::inc], v_av2[::inc, ::inc],
             color='k', alpha=0.5, scale=10, width=0.003)
ax[0].quiver(1230, 50, 1, 0, color='k', scale=10, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f + \overline{\zeta}_2)/\overline{h}_2$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_mean_BEAU' + code + '_L2.png', bbox_inches='tight', dpi=400)
plt.close()

# L3
abs_vort_interp = RegularGridInterpolator((data.xp1.data, data.yp1.data), (f0 + zeta_av[2]).data.T)
abs_vort = abs_vort_interp(np.array([xx.ravel(), yy.ravel()]).T).reshape(yy.shape[0], yy.shape[1]).T
PV = abs_vort/h_av[2]

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     PV, cmap=cmr.rainforest_r,
                     levels=np.linspace(2e-8, 3e-7, 29), extend='both')
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=0.5)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[3990],
              colors='k', linestyles='-', alpha=0.5, linewidths=1)
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3, u_av3[::inc, ::inc], v_av3[::inc, ::inc],
             color='k', alpha=0.5, scale=1, width=0.003)
ax[0].quiver(1230, 50, 0.1, 0, color='k', scale=1, width=0.003, alpha=0.5)
ax[0].text(1300, 80, r'0.1 m s$^{-1}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Potential Vorticity, $(f + \overline{\zeta}_3)/\overline{h}_3$ (m$^{-1}$ s$^{-1}$)', fontsize=10)

class ScalarFormatterForceFormat(ScalarFormatter):
    def _set_format(self):  # Override function that finds format to use.
        self.format = "%1.2f"  # Give format here
        
ax[-1].yaxis.set_major_formatter(ScalarFormatterForceFormat())

plt.savefig('potential_vorticity_mean_BEAU' + code + '_L3.png', bbox_inches='tight', dpi=400)
plt.close()

##############################################################################
# BATHYMETRY AND WIND STRESS #################################################
##############################################################################

f = plt.figure(tight_layout=True, figsize=(6.5, 6))
gs = GridSpec(1, 2, figure=f, width_ratios=[1, 0.03])
ax = []
ax.append(f.add_subplot(gs[0, 0]))
ax.append(f.add_subplot(gs[0, 1]))

f0 = np.sin(np.pi/2)*4*np.pi/(86400)

slope_width = 233.33333

param = {# DOMAIN GEOMETRY
         'res'       :    5e3,     # Resolution in metres
         'Lx'        : 1510e3,     # Domain width in metres
         'Ly'        : 2010e3,     # Domain height in metres
         'GR'        :  750e3,     # Gyre radius in metres
         'GCx'       :  755e3,     # Gyre centre (x coordinate)
         'GCy'       : 1255e3,     # Gyre centre (y coordinate)
         'CRx'       :  250e3,     # Channel radius in metres

         'Lz_min'    :  0.5e3,     # Minimum domain depth in metres
         'Lz_max'    :  4.0e3,     # Maximum domain depth in metres
         'H1'        :  0.8e2,     # Layer 1 depth in metres
         'H2'        :  1.2e2,     # Layer 2 depth in metres
         'noise'     :   True,     # Add noise to bathymetry

         'Sx'        :  slope_width*1e3,     # Slope width in metres

         'layers'    :      3,     # Number of layers

         # WIND FORCING
         'tau_max'   :2.0e-2,     # Maximum wind stress (N/m^2)
         }

def tau_x_poly(X, Y):
    # Wind stress (x component) - polynomial formulation
    # Firstly construct the wind stress within the gyre domain

    # Transform to polar coordinates around the gyre centre
    Xp = X - param['GCx']
    Yp = Y - param['GCy']
    r     = np.sqrt(Xp**2 + Yp**2)
    theta = np.arctan2(Yp, Xp)

    # Define parameters
    b = 1/param['GR']
    d = np.sqrt(2/3)
    a = (4*param['tau_max'])/(d*param['GR']*(2-(b*d*param['GR'])**2))

    # Calculate wind stress
    tau_phi = (a*r/4)*(2-((b**2)*(r**2)))

    tau_x   = tau_phi*np.sin(theta)

    # Now construct the wind stress outside of the gyre
    c = (a*(param['GR']**2)/4)*(2-((b**2)*(param['GR']**2)))

    tau_x_supp = c*(Yp/r**2)
    tau_x[Yp < -param['GR']] = tau_x_supp[Yp < -param['GR']]

    return tau_x

def tau_y_poly(X, Y):
    # Wind stress (y component) - polynomial formulation
    # Firstly construct the wind stress within the gyre domain

    # Transform to polar coordinates around the gyre centre
    Xp = X - param['GCx']
    Yp = Y - param['GCy']
    r     = np.sqrt(Xp**2 + Yp**2)
    theta = np.arctan2(Yp, Xp)

    # Define parameters
    b = 1/param['GR']
    d = np.sqrt(2/3)
    a = (4*param['tau_max'])/(d*param['GR']*(2-(b*d*param['GR'])**2))

    # Calculate wind stress
    tau_phi = (a*r/4)*(2-((b**2)*(r**2)))

    tau_y   = -tau_phi*np.cos(theta)

    # Now construct the wind stress outside of the gyre
    c = (a*(param['GR']**2)/4)*(2-((b**2)*(param['GR']**2)))

    tau_y_supp = -c*(Xp/r**2)
    tau_y[Yp < -param['GR']] = tau_y_supp[Yp < -param['GR']]

    return tau_y

def tau_curl(X, Y):
    # Also plot the vertical component of the wind stress curl 

    taux = tau_x_poly(X, Y)
    tauy = tau_y_poly(X, Y)

    dtauy_dx = np.gradient(tauy, 5000, axis=1)
    dtaux_dy = np.gradient(taux, 5000, axis=0)

    tau_curl = (dtauy_dx - dtaux_dy)

    return tau_curl

tau_u = np.where(h_all > 500, tau_x_poly(xx.T, yy.T), np.nan)
tau_v = np.where(h_all > 500, tau_y_poly(xx.T, yy.T), np.nan)

img = ax[0].contourf(data.x/1e3, data.y/1e3,
                     np.where(h_all > 500, tau_curl(xx.T, yy.T), np.nan), cmap=cmr.ocean,
                     levels=np.linspace(-1e-7, -1e-8, 10), extend='both')
h_inc = [510, 3990]
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=np.arange(490, 4490, 500),
              colors='k', linestyles='--', alpha=0.5, linewidths=1)
ax[0].contour(data.x/1e3, data.y/1e3, h_all, levels=[500, 3990],
              colors='k', linestyles='-', alpha=1, linewidths=2)

inc = 25
ax[0].quiver(data.x[::inc]/1e3, data.y[::inc]/1e3,
             tau_u[::inc, ::inc],
             tau_v[::inc, ::inc],
             color='k', alpha=0.5, scale=0.3, width=0.005)

ax[0].quiver(1250, 50, 2e-2, 0, color='k', scale=0.3, width=0.005)
ax[0].text(1300, 80, r'$2\times 10^{-2}$ N m$^{-2}$', ha='center')

ax[0].set_aspect('equal')
ax[0].set_xlabel('x coordinate (km)')
ax[0].set_ylabel('y coordinate (km)')

for spine in ax[0].spines:
    ax[0].spines[spine].set_color('w')

plt.colorbar(img, cax=ax[-1], orientation='vertical')
ax[-1].tick_params(axis='y', labelsize=10)
ax[-1].set_ylabel(r'Wind stress curl (s$^{-1}$)', fontsize=10)

plt.savefig('config_BEAU' + code + '.png', bbox_inches='tight', dpi=400)
plt.close()



